/***************************************************************************/
/*                                                                         */
/*                              AverageFreqData()                          */
/*                                                                         */
/*		Function to do averaging of frequency data                         */
/*                                                                         */
/*		Parameters:		F_TYPE y[] = frequency data                        */
/*                                   y[0] = # data points                  */
/*									 y[1] = analysis start                 */
/*									 y[2] = analysis end                   */
/*			            int    af  = averaging factor                      */
/*                                 = # points to combine into each average */
/*                                                                         */
/*		Return:         int        = # data points in averaged data,       */
/*									 or -1 if error                        */
/*                                                                         */
/*      Note:           All data points processed regardless of analysis   */
/*                      limits.  Zeros are treated as gaps in freq data.   */
/*						Error if af<1 or af> # original data points.       */
/*                      af=1 is OK, and results in no change in data.      */
/*                                                                         */
/*		Revision record:                                                   */
/*			11/01/92	Created                                            */
/*			02/03/96	Modified for use as Win 3.1 DLL                    */
/*                                                                         */
/* (c) Copyright 1992-6  Hamilton Technical Services  All Rights Reserved  */
/*                                                                         */
/***************************************************************************/

#include "frequenc.h"

int __declspec(dllexport) FAR PASCAL AverageFreqData(F_TYPE y[], int af)
{
	int i;
	int j;
	int k=3;
	int n=(int)y[0];                             /* # original data points */
	int num;
	double sum;

	if( (af<1) || (af>n) )
	{
		return(-1);                                               /* error */
	}

	for(i=3; i+af-1<=n+2; i+=af)
	{
		sum=0;
		num=af;
		for(j=0; j<af; j++)
		{
			if(*(y+i+j))                         /* don't average in a gap */
			{
				sum+=*(y+i+j);
			}                                                    /* end if */
			else
			{
				num--;
			}                                                  /* end else */
		}                                                       /* end for */
		if(num)                                       /* check if all gaps */
		{
			*(y+k++)=sum/num;
		}                                                        /* end if */
		else
		{
			*(y+k++)=0;
		}                                                      /* end else */
	}                                                           /* end for */
	n=k-3;
	*y=n;
	*(y+1)=ceil(*(y+1)/af);
	*(y+2)=floor(*(y+2)/af);

	return(n);                         /* n=# data points in averaged data */
}                                                 /* end AverageFreqData() */

/***************************************************************************/
