/***************************************************************************/
/*                                                                         */
/*					         AveragePhaseData()                            */
/*                                                                         */
/*		Function to average phase data by an integer factor                */
/*                                                                         */
/*		Parameters: 	float x[] = phase data array                       */
/*                                  x[0] = # data points                   */
/*									x[1] = analysis start                  */
/*                                  x[2] = analysis end                    */
/*						int af    = averaging factor                       */
/*                                                                         */
/*		Return:         int       = # points in averaged data,             */
/*									or -1 if error                         */
/*                                                                         */
/*		Note:           All data points are processed regardless of        */
/*                      analysis limits.  Averaging phase data by          */
/*                      factor=af is done by simply using only every       */
/*                      (af)th data point.  A gap is retained if it is     */
/*                      one of the averaged points.  Error if af<1 or      */
/*                      af> # original data points.  af=1 is OK, and       */
/*                      results in no change in the data.                  */
/*                                                                         */
/*		Revision record:                                                   */
/*			12/30/91	Created                                            */
/*			12/31/91	Renamed                                            */
/*          01/16/92    Edited per FrequenC documentation                  */
/*          01/17/92    Edited title block                                 */
/*                      Changed return to int=# points processed           */
/*			01/18/92	Added error trap.                                  */
/*			02/03/96	Modified for use as Win 3.1 DLL                    */
/*                                                                         */
/* (c) Copyright 1991-6  Hamilton Technical Services  All Rights Reserved  */
/*                                                                         */
/***************************************************************************/

#include "frequenc.h"

int __declspec(dllexport) FAR PASCAL AveragePhaseData(F_TYPE x[], int af)

{
	int num=(int)x[0];      /* # original phase data points to be averaged */
	int i;                                /* index for averaged phase data */
			 /* i goes from 1 to ((num-1)/af)+1 averaged phase data points */
	int j;                                /* index for original phase data */
								  /* j starts at the analysis start offset */
		  /* which is 1 for original phase data starting at 1st data point */
							/* j is incremented by the averaging factor af */

	if( (af<1) || (af>num) )
	{
		return(-1);                                               /* error */
	}

	j=1;
	for(i=1; i<=((num-1)/af)+1; i++)
	{
		*(x+i+2) = *(x+j+2);                 /* 2 is the data array offset */
		j += af;
	}                                                           /* end for */
	x[0]=((num-1)/af)+1;
	x[1]=1;
	x[2]=x[0];

	return( (int) x[0]);

}                                                /* end AveragePhaseData() */

/***************************************************************************/
