/***************************************************************************/
/*                                                                         */
/*								Calc_FC                                    */
/*                                                                         */
/*		Program to run some FrequenC functions in calculator mode          */
/*      FrequenC functions used: CalcBias(), CalcBias1(), CalcBias2()      */
/*		CalcBias3(), CalcRatio, DegFree(), CalcNormalProb(),               */
/*		CalcChiSqrProb(), CalcInvChiSqr(), FindPlotScale()                 */
/*		Uses FrequenC functions from FREQUENC.LIB                          */
/*		No parameters (void)                                               */
/*		Returns nothing (void)                                             */
/*                                                                         */
/*		Revision record:                                                   */
/*			02/26/95	Adapted from CALC_FC.C                             */
/*                      For use with disk to accompany "A Test Suite for   */
/*                      the Analysis of Time Domain Frequency Stability",  */
/*                      WJR paper at 1995 IEEE Freq. Contrl. Symp.         */
/*                                                                         */
/*			                                                               */
/*		W. Riley	02/26/95	1230    (c) Hamilton Technical Services	   */
/*																		            */
/***************************************************************************/

#include "frequenc.h"                              /* FrequenC header file */
#include <stdio.h>
#include <conio.h> // ADDED 2/26/95

void main(void)
{
	int code;
	#if(0) // ELIMINATED 2/26/95
	int t;
	#endif
	int num;
	int m;
	int mew;
	int alpha;

	#if(0) // ELIMINATED 2/26/95
	double xx;
	#endif

	float mu;
	float b;
	float r;
	float df;
	float x;
	float p;
	#if(0)// NOT USED 2/26/96
	float chi_sqr;
	#endif

	#if(0) // ELININATED 2/26/95
	F_TYPE min;                                           /* miminum value */
	F_TYPE max;                                           /* maximum value */
	F_TYPE scale_max;                                     /* scale maximum */
	F_TYPE scale_min;                                     /* scale minimum */
	#endif

	while(1)                                               /* loop forever */
	{
		clrscr();
		printf("Calculator Program for FrequenC Functions\n");

		printf("\n  Code   Function         Description");
		printf("\n    0:   CalcBias         Calc B1 bias function"
		" for r=1, =int");
		printf("\n    1:   CalcBias1        Calculate the B1 bias function");
		printf("\n    2:   CalcBias2        Calculate the B2 bias function");
		printf("\n    3:   CalcBias3        Calculate the B3 bias function");
		printf("\n    4:   CalcRatio        Calculate the R(n) function");
		printf("\n    5:   CalcDegFree      Calculate # degrees of freedom");
		printf("\n    6:   CalcNormalProb   Calculate the normal"
		" probability");
		printf("\n    7:   CalcChiSqrProb   Calculate the chi-square"
		" probability");
		printf("\n    8:   CalcInvChiSqr    Calculate the inverse chi-square"
		" value");
		#if(0) // ELIMINATED 2/26/95
		printf("\n    9:   FindPlotScale    Find the plot scale");
		#endif
		printf("\nOther:   Quit             End program");

		printf("\n\nEnter code # for function desired: ");
		num=scanf("%d", &code);
		fflush(stdin);                             /* in case of bad input */
		if(num==0)                                    /* non-integer input */
		{
			printf("\nQuit");
			exit(1);
		}

		switch(code)
		{
			case 0:
			{
				printf("\nCalcBias(N, ):");
				printf("\n    # Samples N = ");
				scanf("%e", &x);
				fflush(stdin);                     /* in case of bad input */
				num=(int) x;
				printf("     (int between -2 to +2) = ");
				scanf("%e", &x);
				fflush(stdin);                     /* in case of bad input */
				mew=(int) x;
				b=CalcBias(num, mew);
				printf("    Bias Function B1 = %e", b);
				if(b==-1)
				{
					printf(" (Error)");
				}
				break;
			}
			case 1:
			{
				printf("\nCalcBias1(N, r, ):");
				printf("\n    # Samples N = ");
				scanf("%e", &x);
				fflush(stdin);                     /* in case of bad input */
				num=(int) x;
				printf("    Dead Time Ratio r = T/ = ");
				scanf("%e", &r);
				fflush(stdin);                     /* in case of bad input */
				printf("     (-2 to +2) = ");
				scanf("%e", &mu);
				fflush(stdin);                     /* in case of bad input */
				b=CalcBias1(num, r, mu);
				printf("    Bias Function B1 = %e", b);
				if(b==-1)
				{
					printf(" (Error)");
				}
				break;
			}
			case 2:
			{
				printf("\nCalcBias2(r, ):");
				printf("\n    Dead Time Ratio r = T/ = ");
				scanf("%e", &r);
				fflush(stdin);                     /* in case of bad input */
				printf("     (-2 to +2) = ");
				scanf("%e", &mu);
				fflush(stdin);                     /* in case of bad input */
				b=CalcBias2(r, mu);
				printf("    Bias Function B2 = %e", b);
				if(b==-1)
				{
					printf(" (Error)");
				}
				break;
			}
			case 3:
			{
				printf("\nCalcBias3(m, r, ):");
				printf("\n    Averaging Factor m = ");
				scanf("%e", &x);
				fflush(stdin);                     /* in case of bad input */
				m=(int) x;
				printf("    Dead Time Ratio r = T/ = ");
				scanf("%e", &r);
				fflush(stdin);                     /* in case of bad input */
				printf("     (-2 to +2) = ");
				scanf("%e", &mu);
				fflush(stdin);                     /* in case of bad input */
				b=CalcBias3(m, r, mu);
				printf("    Bias Function B3 = %e", b);
				if(b==-1)
				{
					printf(" (Error)");
				}
				break;
			}
			case 4:
			{
				printf("\nCalcRatio(, n, BW):");
				printf("\n     (int between -2 to +2) = ");
				scanf("%e", &x);
				fflush(stdin);                     /* in case of bad input */
				alpha=(int) x;                     /* alpha must be an int */
				printf("    # Phase Data Points Averaged n = ");
				scanf("%e", &x);
				fflush(stdin);                     /* in case of bad input */
				num=(int) x;                         /* num must be an int */
				if(alpha==1)
				{
					printf("    Bandwidth Factor BW = 2fh0 = ");
					scanf("%e", &b);
					fflush(stdin);                     /* in case of bad input */
				}
				else
				{
					b=1.0;
				}
				r=CalcRatio(alpha, num, b);
				printf("    Ratio R(n) = %e", r);
				if(r==-1)
				{
					printf(" (Error)");
				}
				break;
			}
			case 5:
			{
				printf("\nCalcDegFree(, N, m):");
				printf("\n     (int between -2 to +2) = ");
				scanf("%e", &x);
				fflush(stdin);                     /* in case of bad input */
			alpha=(int) x;                     /* alpha must be an int */
				printf("    # Phase Data points N = ");
				scanf("%e", &x);
				fflush(stdin);                     /* in case of bad input */
				num=(int) x;                         /* num must be an int */
				printf("    Averaging Factor m = ");
				scanf("%e", &x);
				fflush(stdin);                     /* in case of bad input */
				m=(int) x;                             /* m must be an int */
				df=CalcDegFree(alpha, num, m);
				printf("    # Degrees of Freedom df = %e", df);
				if(df==-1)
				{
					printf(" (Error)");
				}
				break;
			}
			case 6:
			{
				printf("\nCalcNormalProb(x):");
				printf("\n    Normal Deviate x = ");
				scanf("%e", &x);
				fflush(stdin);                     /* in case of bad input */
				p=CalcNormalProb(x);
				printf("    Normal Probability p = %e", p);
				break;
			}
			case 7:
			{
				printf("\nCalcChiSqrProb(X, df):");
				printf("\n    Chi-Square X = ");
				scanf("%e", &x);
				fflush(stdin);                     /* in case of bad input */
				printf("    # Degrees of Freedom df = ");
				scanf("%e", &df);
				fflush(stdin);                     /* in case of bad input */
				p=CalcChiSqrProb(x, df);
				printf("    Chi-Square Probability p = %e", p);
				if(p==-1)
				{
					printf(" (Error)");
				}
				break;
			}
			case 8:
			{
				printf("\nCalcInvChiSqr(p, df):");
				printf("\n    Chi-Square Probability p = ");
				scanf("%e", &p);
				fflush(stdin);                     /* in case of bad input */
				printf("    # Degrees of Freedom df = ");
				scanf("%e", &df);
				fflush(stdin);                     /* in case of bad input */
				x=CalcInvChiSqr(p, df);
				printf("    Chi-Square X = %e", x);
				if(x==-1)
				{
					printf(" (Error)");
				}
				break;
			}
			#if(0) // ELIMINATED 2/26/95
			case 9:
			{
				printf("\nFindPlotScale(Min, Max):");
				printf("\n    Data Minimum = ");
				scanf("%le", &xx);
				fflush(stdin);                     /* in case of bad input */
				min=(F_TYPE) xx;
				printf("    Data Maximum = ");
				scanf("%le", &xx);
				fflush(stdin);                     /* in case of bad input */
				max=(F_TYPE) xx;
				t=FindPlotScale(min, max, &scale_min, &scale_max);
				printf("    Scale Minimum = %17.10e", scale_min);
				printf("\n    Scale Maximum = %17.10e", scale_max);
				printf("\n    # Tick Intervals = %d", t);
				if(t==-1)
				{
					printf(" (Error)");
				}
				break;
			}
			#endif
			default:                         /* integer input not 0 thru 9 */
			{
				printf("\nQuit");
				exit(1);
			}
		}                                                    /* end switch */

	printf("\n\nPress any key to continue...");
	getch();

	}                                                         /* end while */
}                                                         /* end test_fc() */

/***************************************************************************/
