/***************************************************************************/
/*                                                                         */
/*  						CalcLinFreqDrift()                             */
/*                                                                         */
/*		Function to calculate linear frequency drift y(t)=a+bt             */
/*                                                                         */
/*		Parameters:		F_TYPE y[]  = frequency data                       */
/*                                    y[0] = # data points                 */
/*			                          y[1] = analysis start                */
/*                                    y[2] = analysis end                  */
/*      	            F_TYPE p_a  = pointer to y-intercept               */
/*      	            F_TYPE p_b  = pointer to slope                     */
/*			            F_TYPE p_v  = pointer to fit variance              */
/*                                                                         */
/*      Return:         int         = # analysis points,                   */
/*                                    or -1 if error                       */
/*                                                                         */
/*	    Notes:	        Zeros treated as gaps in frequency data	           */
/*						# non-gap data points must be at least 2           */
/*						and at least 3 to calc finite fit variance         */
/*                                                                         */
/*		Revision record:                                                   */
/*			12/16/91	Created                                            */
/*			12/26/91	Changed to int returning # non-gap data points     */
/*			12/31/91	Renamed                                            */
/*          01/16/92    Edited per FrequenC documentation                  */
/*			01/18/92	Edited title block                                 */
/*			01/19/92	Added error traps                                  */
/*			02/03/96	Modified for use as Win 3.1 DLL                    */
/*			01/01/98	Changes for MS VC++ compatibility & warnings       */
/*                                                                         */
/* (c) Copyright 1991-8  Hamilton Technical Services  All Rights Reserved  */
/*                                                                         */
/***************************************************************************/

#include "frequenc.h"

int __declspec(dllexport) FAR PASCAL CalcLinFreqDrift(F_TYPE y[],
	F_TYPE *p_a, F_TYPE *p_b, F_TYPE *p_v)
{
	int i;
	int num = (int)(*(y+2)-*(y+1)+1);
	double sum_x = 0;
	double sum_y = 0;
	double sum_xx = 0;
	double sum_yy = 0;
	double sum_xy = 0;

	for( i = (int)(*(y+1)+2); i <= (int)(*(y+2)+2); i++)
	{
		if(*(y + i))
		{
			sum_x  += i-2;
			sum_y  += *(y + i);
			sum_xx += SQR ( (float) (i-2) );         /* must cast to float */
			sum_yy += SQR ( *(y + i) );
			sum_xy += (i-2) * *(y + i);
		}                                                        /* end if */
		else
		{
			num--;
		}                                                      /* end else */
	}                                                           /* end for */

	if(num<2)
	{
		return(-1);                                               /* error */
	}

	*p_b = (sum_xy - ((sum_x) * (sum_y) / num));
	*p_b = *p_b / (sum_xx- ((sum_x) * (sum_x)) / num);
	*p_a = ((sum_y) / num) - (*p_b * sum_x / num);
	*p_v = 0;                           /* fit variance 0 if only 2 points */

	if(num>=3)     /* need at least 3 analysis points to calc fit variance */
	{
		*p_v = sum_yy  - (sum_y * sum_y / num);
		*p_v -= *p_b * *p_b * (sum_xx - (sum_x * sum_x / num ));
		*p_v /= (num - 2);
	}

	return(num);
}                                                /* end CalcLinFreqDrift() */

/***************************************************************************/
