/***************************************************************************/
/*                                                                         */
/*                 			FillGaps()                                     */
/*                                                                         */
/*		Function to fill gaps in data with interpolated values             */
/*                                                                         */
/*		Parameters:  	F_TYPE z[]  = phase or frequency data              */
/*                                    z[0] = # data points                 */
/*			                          z[1] = analysis start                */
/*                                    z[2] = analysis end                  */
/*			            BOOL datype = data type: 0=phase, 1=frequency      */
/*                                                                         */
/*		Return:         int         = # gaps removed                       */
/*	                                                                       */
/*      Note:           All zeros treated as gaps in frequency data.       */
/*                      Embedded zeros treated as gaps in phase data.      */
/*                                                                         */
/*      Revision record:                                                   */
/*  		12/16/91	Created                                            */
/*			12/24/91	Changed to make datype a parameter rather than a   */
/*						global variable.  Remove call to inc_extension().  */
/*			12/31/91	Renamed                                            */
/*          01/18/92    Edited title                                       */
/*			07/22/93	Fixed bug that changed i inside interior gap loop  */
/*			02/03/96	Modified for use as Win 3.1 DLL                    */
/*			01/01/98	Changes for MS VC++ compatibility & warnings       */
/*                                                                         */
/* (c) Copyright 1991-8  Hamilton Technical Services  All Rights Reserved  */
/*                                                                         */
/***************************************************************************/

#include "frequenc.h"

int __declspec(dllexport) FAR PASCAL FillGaps(F_TYPE z[], BOOL datype)

{
	int g = 0;                                          /* total gap count */
	int i;                                        /* general (outer) index */
	int i_start;               /* index of last data point at start of gap */
	int i_end;                   /* index of next data point at end of gap */
	int j;                                                  /* inner index */
	int k;                                              /* local gap count */
	int n = (int)*z;                                      /* # data points */
	double increment;              /* freq increment for interpolated data */

	for(i=(int)(*(z+1)+3-datype); i<=(int)(*(z+2)+1+datype); i++)
		                                              /* elim leading gaps */
	{
		if(!*(z+i))                                /* is data point a gap? */
		{
			for(j=i; j<n+2; j++)         /* move all data up to remove gap */
			{
				*(z+j) = *(z+j+1);
			}                                                   /* end for */
			i--;   /* after shifting, must test the first data point again */
			g++;                          /* increment the total gap count */
			n--;                              /* decrement the data counts */
			*z=n;
			*(z+2)=*(z+2)-1;
		}                                                        /* end if */
		else
		{
			break;        /* stop when first non-gap data point is reached */
		}                                                      /* end else */
	}                                                           /* end for */

	for(i=(int)(*(z+2)+1+datype); i>=(int)(*(z+1)+3-datype); i--)
		                                             /* elim trailing gaps */
	{
		if(!*(z+i))                                /* is data point a gap? */
		{
			for(j=i; j<n+2; j++)         /* move all data up to remove gap */
			{
				*(z+j) = *(z+j+1);
			}                                                   /* end for */
			g++;                          /* increment the total gap count */
			n--;                              /* decrement the data counts */
			*z=n;
			*(z+2)=*(z+2)-1;
		}                                                        /* end if */
		else
		{
			break;         /* stop when last non-gap data point is reached */
		}
	}                                                           /* end for */

	k=0;                 /* replace interior gaps with interpolated values */
	for(i=(int)(*(z+1)+3-datype); i<=(int)(*(z+2)+1+datype); i++)
	{
		if(!*(z+i))                                /* is data point a gap? */
		{
			k++;                          /* increment the local gap count */
		}                                                        /* end if */
		else
		{
			if(k)                                      /* are we at a gap? */
			{
				i_end = i;        /* save the index for the end of the gap */
				i_start = i-k-1;    /* save index for the start of the gap */
				increment = (*(z+i_end) - *(z+i_start)) / (k+1);
				for(j=1; j<=k; j++)
				{
					*(z+i_start+j) = *(z+i_start) + increment*j;
					g++;                        /* increment the gap count */
				}                                               /* end for */
			k=0;                        /* reinitialize count for next gap */
			}                                                    /* end if */
		}                                                      /* end else */
	}                                                           /* end for */

	return(g);                              /* return total # gaps removed */
}                                                        /* end FillGaps() */

/***************************************************************************/
