/***************************************************************************/
/*                                                                         */
/* 							 RemoveQuadraticDrift()                        */
/*																		   */
/*		Function to remove frequency drift from phase data                 */
/*		by removing quadratic fit x(t) = aln(bt+1) + c                    */
/*                                                                         */
/*		Parameters:		F_TYPE x[] = phase data                            */
/*                                   x[0] = # phase data points            */
/*			                         x[1] = analysis start                 */
/*                                   x[2] = analysis end                   */
/*                      F_TYPE a  =  multiplicative term                   */
/*                      F_TYPE b  =  log term                              */
/*                      F_TYPE c  =  constant term                         */
/*                                                                         */
/*      Return:         int       = # data points processed                */
/*                                                                         */
/*    	Notes:          Embedded zeros treated as gaps in phase data.      */
/*		                Use FrequenC function CalcQuadraticDrift() to      */
/*                      calculate quadratic fit parameters a, b and c.     */
/*                                                                         */
/*		Revision record:                                                   */
/*			12/16/90	Created                                            */
/*			12/31/91	Renamed                                            */
/*			01/18/92	Edited title block                                 */
/*			02/03/96	Modified for use as Win 3.1 DLL                    */
/*                                                                         */
/* (c) Copyright 1990-6  Hamilton Technical Services  All Rights Reserved  */
/*                                                                         */
/***************************************************************************/

#include "frequenc.h"

int __declspec(dllexport) RemoveQuadraticDrift(F_TYPE x[], F_TYPE a,
	F_TYPE b, F_TYPE c)
{
	int i;                                                        /* index */
	int num=0;                                       /* # points processed */

	for( i = *(x+1)+2; i <= *(x+2)+2; i++)         /* remove quadratic fit */
	{
		if(*(x+i))                                        /* check for gap */
		{
			*(x+i) -=  a + (b * i) + (c * i * i);      /* remove quad term */
			num++;                                      /* increment count */
		}                                                        /* end if */
	}                                                           /* end for */
	return(num);
}                                            /* end RemoveQuadraticDrift() */

/***************************************************************************/
