/***************************************************************************/
/*                                                                         */
/*          Starter Win32 Console Program to Test FrequenC Library         */
/*                                                                         */
/*                                                                         */
/*  Revision record:                                                       */
/*			05/06/08    Created                                            */
/*                                                                         */
/* (c) 2008  W. Riley  Hamilton Technical Services  All Rights Reserved    */
/*                                                                         */
/***************************************************************************/

// Includes
#include <Windows.h>
#include "FrequenC.h"

// Macro for F_TYPE data type
#define F_TYPE double

// Macro for FrequenC library version number
#define FREQUENC_DLL_VERSION_NUMBER 300

// DLL Handle
HMODULE hFCLib;			// FrequenC DLL module handle

int main(void)
{
	// Create a set of classic freq test data as a static F_TYPE array
	// See NBS Monograph 140 Annex8.E
	// This array includes header info: # pts, analy start, analy end
	// and 9 freq data points
	F_TYPE fDat[] = {9, 1, 9, 892, 809, 823, 798, 671, 644, 883, 903, 677};

	// Local variables
	int nNum;			// # analysis points
	int nVer;			// FrequenC version code
	F_TYPE fSig=1.0;	// Sigma

	// Show heading
	printf("FrequenC Library Starter Example Program\n\n");

	// Load 32-bit FrequenC DLL
	hFCLib=LoadLibrary("FREQUENC.DLL");

	// Check if FrequenC DLL loaded OK
	if((UINT)hFCLib<=32)
	{
		// Show error message
		printf("Unable to Load FrequenC DLL\n");
		
		// End program
		return 1;
	}
	else // OK
	{
		// Show success message
		printf("FrequenC DLL Loaded\n");
	}		

	// Check FrequenC DLL version #
	if((nVer=CheckFrequenC())!=FREQUENC_DLL_VERSION_NUMBER)
	{
		// Show error message
		printf("Invalid FrequenC DLL Version\n");
		
		// End program
		return 1;
	}
	else // OK
	{
		// Show FrequenC version
		printf("FrequenC Version = %3.2f\n", nVer/100.);
	}		

	// Call the Allan deviation function for test data array
	nNum=CalcFreqSigma(fDat, &fSig);

	// Show results message
	printf("\nAllan Deviation Results:\n");

	// Display the # analysis points
	printf("# Analysis Points= %d\n", nNum);

	// Show sigma result
	printf("Sigma = %e\n\n", fSig);

	// Done
	return 0;
}

