/*****************************************************************************/
/*                                                                           */
/*                            TEST_DLL.C                                      */
/*                                                                           */
/*		Program to display a basic window under Microsoft Windows 3.1        */
/*		Uses CalcNormalProb() function in FrequenC DLL                                             */
/*                                                                           */
/*		Operating Environment:		MS-DOS 6.2 and Windows 3.1               */
/*		Programming Environment:	Borland C++ 4.52                         */
/*		Programming Language:		Standard C                               */
/*		Project Directory:			D:\BC45\FREQUENC                          */
/*		Project Name:				TEST_DLL                                  */
/*		Project Files:				TEST_DLL.IDE 	Project IDE File         */
/*									TEST_DLL.C		Source File              */
/*									TEST_DLL.DEF    Module Definition File   */
/*									FREQUENC.DLL    FrequenC DLL                  */
/*		Results:					Project compiles and runs AOK            */
/*                                                                           */
/*		Programmer:					W. Riley                                 */
/*		Date:						02/03/96                                 */
/*                                                                           */
/*****************************************************************************/

// Include the Windows header file
#include <windows.h>

// Standard C header files
#include <stdio.h>

// Declare WndProc() function
long FAR PASCAL _export WndProc(HWND, UINT, UINT, LONG);

// Declare the DLL function
extern float FAR PASCAL _export CalcNormalProb(float);

// Windows main entry function
// Returns integer
// Uses PASCAL calling sequence
// HANDLE hInstance is ID # of running program
// HANDLE hPrevInstance is ID # of previous instance of program (if any -
// NULL if no previous instance is still running)
// LPSTR lpszCmdParam is command line argument string - not used
// int nCmdShow is # indicating how window is to be initially shown
int PASCAL WinMain(HANDLE hInstance, HANDLE hPrevInstance,
						  LPSTR lpszCmdParam, int nCmdShow)
	{
	// Define variables
	static char szAppName[] = "TEST_DLL";	// Main window classname
	HWND        hwnd;						// Window handle
	MSG         msg;						// Message
	WNDCLASS    wndclass;					// Window class

	if(!hPrevInstance) // Register class only once
	{
		// Fill WNDCLASS structure parameters
		wndclass.style         = CS_HREDRAW | CS_VREDRAW;
		wndclass.lpfnWndProc   = WndProc;
		wndclass.cbClsExtra    = 0;
		wndclass.cbWndExtra    = 0;
		wndclass.hInstance     = hInstance;
		wndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION);
		wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW);
		wndclass.hbrBackground = COLOR_BACKGROUND;
		wndclass.lpszMenuName  = NULL;
		wndclass.lpszClassName = szAppName;

		// Register window class
		RegisterClass(&wndclass);
	}

	// Create main window
	hwnd = CreateWindow(szAppName,           		// window class name
						"Test FrequenC DLL",		// window caption
						WS_OVERLAPPEDWINDOW,     	// window style
						CW_USEDEFAULT,    		    // initial x position
						CW_USEDEFAULT,				// initial y position
						CW_USEDEFAULT,				// initial x size
						CW_USEDEFAULT,				// initial y size
						NULL,					   	// parent window handle
						NULL,						// window menu handle
						hInstance,					// program instance handle
						NULL);		                // creation parameters


	// Put window on screen
	ShowWindow(hwnd, nCmdShow);
	// Paint client area
	UpdateWindow(hwnd);

	// Message loop
	while(GetMessage(&msg, NULL, 0, 0))
	{
		// Do keyboard translation
		TranslateMessage(&msg);
		// Pass message back to Windows
		DispatchMessage(&msg);
	}
	return msg.wParam;
}

// Window procedure
long FAR PASCAL _export WndProc(HWND hwnd, UINT message, UINT wParam,
								LONG lParam)
{
	// Local variables
	HDC hdc;
	PAINTSTRUCT ps;
	RECT rect;
	float arg;
	float val;
	char str[16];

	// Main message switching procedure
	switch(message)
	{
		case WM_PAINT:
			hdc=BeginPaint(hwnd, &ps);
			GetClientRect(hwnd, &rect);

			arg=0.5;
			val=CalcNormalProb(arg); // Get value from DLL function
			sprintf(str, "p(%f)=%f", arg, val);

			DrawText(hdc, str, -1, &rect,
					 DT_SINGLELINE | DT_CENTER | DT_VCENTER);
			EndPaint(hwnd, &ps);
			return 0;

		case WM_DESTROY:
			PostQuitMessage(0); // Tell Windows app is terminating
			return 0;
	}
	// Call DefWindowProc to process other messages
	return DefWindowProc(hwnd, message, wParam, lParam);
}
