/***************************************************************************/
/*                                                                         */
/*                          FrequenC.h                                     */
/*                                                                         */
/*      Header file for FrequenC Library                                   */
/*      To be included in all FrequenC source files                        */
/*                                                                         */
/*                                                                         */
/*      Revision record:                                                   */
/*          12/24/91    Created                                            */
/*          12/25/91    More drafting                                      */
/*          12/26/91    Ditto                                              */
/*          12/29/91    Ditto                                              */
/*          12/30/91    Ditto                                              */
/*          12/31/91    Functions renamed                                  */
/*          01/04/92    All CalcDegFree() parameters made ints             */
/*          01/18/92    Added a & b parameters to RemoveLinFreqDrift()     */
/*                      Added a, b & c params to RemoveQuadraticDrift()    */
/*          01/19/92    Changed CalcSecondDiff() to int and added *p_d     */
/*          01/22/92    Changed order of parameters in FindPlotScale()     */
/*          01/26/92    Added #defines for PSD and SIG types               */
/*          02/17/92    Changed min & max params in FindPlotScale()        */
/*          02/18/92    Added #defines for noise types                     */
/*                      Eliminated unneeded #defines & #include files      */
/*          02/20/92    Eliminated #include <values.h> -- new FMM.C        */
/*          11/28/92    Added prototypes for the following new functions:  */
/*                          CalcHadamardDev()                              */
/*                          CalcGreenhallModSigma()                        */
/*                          CalcThreePointDrift()                          */
/*                          CalcBisectionDrift()                           */
/*                          FindFreqOutliers()                             */
/*                          MJDToDate()                                    */
/*                          DateToMJD()                                    */
/*          04/03/93    Added stdio.h include file                         */
/*          02/03/96    Modified for use as Win 3.1 DLL                    */
/*          01/01/98    Added EXPORT macro for Win 16/32 portability       */
/*          03/08/03    Changed CalcPhaseModSigma() prototype              */
/*                      Changed ConvPhaseToFreq() prototype                */
/*          03/09/03    Brought in many functions from Stable_r.c          */
/*          03/11/03    Added date functions                               */
/*			03/14/03	New FrequenC dll working with Stable32             */
/*                                                                         */
/* (c)Copyright 1991-2003 Hamilton Technical Services All Rights Reserved  */
/*                                                                         */
/***************************************************************************/

/* Windows include file */
#include <windows.h>

/* Standard C include files */
#include <math.h>
#include <stdlib.h>
#include <stdio.h>

/* definitions */
#define SQR( a)  ((a) * (a))                         /* macro for squaring */
#define FALSE 0                                              /* BOOL value */
#define TRUE 1                                               /* BOOL value */
#define F_TYPE double                           /* can be changed to float */
#define BOOL int                                              /* BOOL type */
#define PHASE 0                                 /* datype=0 for phase data */
#define PHASE_DATA 0                            /* datype=0 for phase data */
#define FREQ 1                              /* datype=1 for frequency data */
#define FREQ_DATA 1                         /* datype=1 for frequency data */
#define PSD_Y 1                                    /* type=1 for Sy(f) PSD */
#define PSD_X 2                                    /* type=2 for Sx(f) PSD */
#define PSD_P 3                                    /* type=3 for S(f) PSD */
#define PSD_L 4                                    /* type=4 for L(f)  PSD */
#define SIG_Y 5                                        /* type=5 for y() */
#define MODSIG_Y 6                                 /* type=6 for Mod y() */
#define SIG_X 7                                        /* type=7 for x() */
#define W_PM 2                                /* noise type=2 for white PM */
#define F_PM 1                              /* noise type=1 for flicker PM */
#define W_FM 0                                /* noise type=0 for white FM */
#define F_FM -1                            /* noise type=-1 for flicker FM */
#define RW_FM -2                       /* noise type=-2 for random walk FM */
#define ARRAY_OFFSET 3                         /* Offset to 1st data point */

// Sigma Types
#define NORMAL_SIGMA           0
#define OVERLAPPING_SIGMA      1
#define MODIFIED_SIGMA         2
#define TIME_SIGMA             3
#define HADAMARD_SIGMA         4
#define OVERLAPHAD_SIGMA       5
#define TOTAL_SIGMA            6
#define MODTOTAL_SIGMA         7
#define TIMETOTAL_SIGMA        8
#define HADTOTAL_SIGMA         9
#define TIERMS_SIGMA           10
#define MTIE_SIGMA             11
#define THEO1_SIGMA            12

/* FrequenC function prototypes */ 

// Variance functions
int __declspec(dllexport) FAR PASCAL CalcFreqSigma(F_TYPE y[], F_TYPE *p_sig);
int __declspec(dllexport) FAR PASCAL CalcFreqStdDev(F_TYPE y[],
    F_TYPE *p_dev);
int __declspec(dllexport) FAR PASCAL CalcFreqModSigma(F_TYPE y[],
    F_TYPE *p_modsig, int m);
int __declspec(dllexport) FAR PASCAL CalcFreqOverlapSigma(F_TYPE y[],
    F_TYPE *p_sig, int m);
int __declspec(dllexport) FAR PASCAL CalcFastModSigma(F_TYPE x[],
    F_TYPE *p_modsig, F_TYPE tau, int m);
int __declspec(dllexport) FAR PASCAL CalcPhaseStdDev(F_TYPE x[],
    F_TYPE *p_dev, F_TYPE tau);
int __declspec(dllexport) FAR PASCAL CalcPhaseModSigma(F_TYPE x[],
    F_TYPE *p_sig, F_TYPE tau, int m, BOOL *bAbort, BOOL bProgress,
    void *ProgressFunction(int nPercentDone, char *szMessage));
int __declspec(dllexport) FAR PASCAL CalcPhaseOverlapSigma(F_TYPE x[],
    F_TYPE *p_sig, F_TYPE tau, int m);
int __declspec(dllexport) FAR PASCAL CalcPhaseSigma(F_TYPE x[],
    F_TYPE *p_sig, F_TYPE tau);
int __declspec(dllexport) FAR PASCAL CalcPhaseHadamardDev(F_TYPE x[],
    F_TYPE *p_dev, F_TYPE tau);
int __declspec(dllexport) FAR PASCAL CalcGreenhallModSigma(F_TYPE x[],
    F_TYPE *p_sig, F_TYPE tau, int m);
int __declspec(dllexport) FAR PASCAL CalcHadamardDev(F_TYPE x[],
    F_TYPE *p_dev, F_TYPE tau);
int __declspec(dllexport) FAR PASCAL CalcFreqOverlapHadamardDev(F_TYPE y[],
    F_TYPE *p_sig, int m);
int __declspec(dllexport) FAR PASCAL CalcFreqOverlapSigma(F_TYPE y[],
    F_TYPE *p_sig, int m);
int __declspec(dllexport) FAR PASCAL CalcMTIE(F_TYPE fX[], F_TYPE *fMTIE,
    int nAF, BOOL *bAbort, BOOL bProgress,
    void *ProgressFunction(int nPercentDone, char *szMessage));
int __declspec(dllexport) FAR PASCAL CalcPhaseModSigma(F_TYPE x[],
    F_TYPE *p_sig, F_TYPE tau, int m, BOOL *bAbort, BOOL bProgress,
    void *ProgressFunction(int nPercentDone, char *szMessage));
int __declspec(dllexport) FAR PASCAL CalcPhaseOverlapHadamardDev(F_TYPE x[],
    F_TYPE *p_dev, F_TYPE tau, int m);
int __declspec(dllexport) FAR PASCAL CalcPhaseStdDev(F_TYPE x[],
    F_TYPE *p_dev, F_TYPE tau);
int __declspec(dllexport) FAR PASCAL TotvarCalc(F_TYPE fX[], F_TYPE *fTotDev,
    F_TYPE fTau, int nAF, void *lpfnProgress);
int __declspec(dllexport) FAR PASCAL CalcFreqHadamardDev(F_TYPE y[],
    F_TYPE *p_dev);
int __declspec(dllexport) FAR PASCAL ModTotvarCalc(F_TYPE fX[],
    F_TYPE *fMTDev, F_TYPE fTau, int nAF, BOOL *bAbort, BOOL bProgress,
    void *ProgressFunction(int nPercentDone, char *szMessage));
int __declspec(dllexport) FAR PASCAL TIErms(F_TYPE fX[], F_TYPE *fTIE,
    int nAF);
int __declspec(dllexport) FAR PASCAL HadTotvarCalc(F_TYPE fY[], F_TYPE *fHTDev,
    int nAF, BOOL *bAbort, BOOL bProgress,
    void *ProgressFunction(int nPercentDone, char *szMessage));

// EDF functions
float __declspec(dllexport) FAR PASCAL EDF(int nBeta, int nAvgFactor, int nNum);
float __declspec(dllexport) FAR PASCAL CalcDegFree(int a, int n, int m);
float __declspec(dllexport) FAR PASCAL HadamardEDF(int N, int m, int b);
float __declspec(dllexport) FAR PASCAL HadTotvarEDF(int nAlpha,
    float fRatio);
float __declspec(dllexport) FAR PASCAL TotvarEDF(int nAlpha, float fRatio);
float __declspec(dllexport) FAR PASCAL ModTotvarEDF(int nAlpha,
    float fRatio);

// Bias functions
float __declspec(dllexport) FAR PASCAL CalcRatio(int alpha, int num,
    float b);
float __declspec(dllexport) FAR PASCAL CalcBias(int num, int mu);
float __declspec(dllexport) FAR PASCAL CalcBias1(int n, float r, float mu);
float __declspec(dllexport) FAR PASCAL CalcBias2(float r, float mu);
float __declspec(dllexport) FAR PASCAL CalcBias3(int m, float r, float mu);
float __declspec(dllexport) FAR PASCAL TotvarBias(int nAlpha, float fRatio);
float __declspec(dllexport) FAR PASCAL HadTotvarBias(int nAlpha);
float __declspec(dllexport) FAR PASCAL CalcHadamardB1(int num, int mu);
float __declspec(dllexport) FAR PASCAL CalcStarB1(F_TYPE fData[],
    BOOL bDataType, F_TYPE fTau, int nAF);
float __declspec(dllexport) FAR PASCAL ModTotvarBias(int nAlpha);

// Drift functions
int __declspec(dllexport) FAR PASCAL CalcLinFreqDrift(F_TYPE y[],
    F_TYPE *p_a, F_TYPE *p_b, F_TYPE *p_v);
int __declspec(dllexport) FAR PASCAL CalcLogFreqDrift(F_TYPE y[],
    F_TYPE *p_a, F_TYPE *p_b, F_TYPE *p_c, F_TYPE *p_v);
int __declspec(dllexport) FAR PASCAL RemoveLinFreqDrift(F_TYPE y[], F_TYPE a,
    F_TYPE b);
int __declspec(dllexport) FAR PASCAL RemoveLogFreqDrift(F_TYPE y[], F_TYPE a,
    F_TYPE b, F_TYPE c);
int __declspec(dllexport) FAR PASCAL RemoveQuadraticDrift(F_TYPE x[],
    F_TYPE a, F_TYPE b, F_TYPE c);
int __declspec(dllexport) FAR PASCAL CalcQuadraticDrift(F_TYPE x[], F_TYPE t,
    F_TYPE *p_a, F_TYPE *p_b, F_TYPE *p_c, F_TYPE *p_v);
int __declspec(dllexport) FAR PASCAL RemoveDiffusionFreqDrift(F_TYPE y[],
    F_TYPE a, F_TYPE b, F_TYPE c);
float __declspec(dllexport) FAR PASCAL CalcThreePointDrift(F_TYPE x[]);
float __declspec(dllexport) FAR PASCAL CalcBisectionDrift(F_TYPE y[]);
int __declspec(dllexport) FAR PASCAL CalcDiffusionFreqDrift(F_TYPE y[],
    F_TYPE *p_a, F_TYPE *p_b, F_TYPE *p_c, F_TYPE *p_v);
int __declspec(dllexport) FAR PASCAL CalcSecondDiff(F_TYPE x[], F_TYPE *p_d);
  
// Misc statistics functions
int __declspec(dllexport) FAR PASCAL CalcMean(F_TYPE z[], F_TYPE *p_avg,
    BOOL datype);
int __declspec(dllexport) FAR PASCAL FindMinMax(F_TYPE z[], F_TYPE *p_min,
    F_TYPE *p_max, BOOL datype);
int __declspec(dllexport) FAR PASCAL FindMedian(F_TYPE fData[],
    BOOL bDataType, F_TYPE *pfMedian);
int __declspec(dllexport) FAR PASCAL MedDev(F_TYPE y[], float *p_mad);
void __declspec(dllexport) FAR PASCAL HistoCalc(F_TYPE y[], float bins[],
	float hist[], F_TYPE *y_max, int num_bins, BOOL bDataType);
   
// Plot scale functions 
int __declspec(dllexport) FAR PASCAL FindPlotScale(F_TYPE min, F_TYPE max,
    F_TYPE *p_scale_min, F_TYPE *p_scale_max);
int  __declspec(dllexport) FAR PASCAL BasScale (double XMin, double XMax,
	int N, double* SMin, double* Step);
void  __declspec(dllexport) FAR PASCAL RoundAxes(F_TYPE *a1, F_TYPE *a2,
    F_TYPE *tics);
   
// Data conversion functions
int __declspec(dllexport) FAR PASCAL ConvFreqToPhase(F_TYPE y[], F_TYPE x[],
    F_TYPE tau, BOOL do_norm);
int __declspec(dllexport) FAR PASCAL ConvPhaseToFreq(F_TYPE x[], F_TYPE y[],
    F_TYPE tau, BOOL z); 
int __declspec(dllexport) FAR PASCAL NewConvFreqToPhase(F_TYPE y[],
    F_TYPE x[], F_TYPE tau, BOOL do_norm);
int  __declspec(dllexport) FAR PASCAL ConvPhaseToFreqUsingTimetags(F_TYPE x[],
	F_TYPE y[], F_TYPE t[], F_TYPE scale, F_TYPE *avg, BOOL z);
   
// Domain conversion functions
float __declspec(dllexport) FAR PASCAL AddSigma(float fSigma[]);
float __declspec(dllexport) FAR PASCAL AddPSD(float fPSD[], int nPSDType);
int __declspec(dllexport) FAR PASCAL CalcDomain(float *p_h, float z,
    float tau, float f, float v, float fh, int type, int alpha, int n);
int __declspec(dllexport) FAR PASCAL ConvDomain(float h, float *p_z,
	float tau, float f, float v, float fh, int type, int alpha, int n);
    
// Gap handling functions
int __declspec(dllexport) FAR PASCAL FillGaps(F_TYPE z[], BOOL datype);
int __declspec(dllexport) FAR PASCAL FillFloatGaps(float z[],
	BOOL datype);
int __declspec(dllexport) FAR PASCAL CountGaps(F_TYPE z[], BOOL datype);

// Probability functions
float __declspec(dllexport) FAR PASCAL CalcInvChiSqr(float p, float df);
float __declspec(dllexport) FAR PASCAL CalcNormalProb(float x);
float __declspec(dllexport) FAR PASCAL CalcChiSqrProb(float x, float df);

// Utility functions
int __declspec(dllexport) FAR PASCAL CheckFrequenC(void);

// Data manipulation functions
int __declspec(dllexport) FAR PASCAL AverageFreqData(F_TYPE y[],
    int avg_factor);
int __declspec(dllexport) FAR PASCAL AveragePhaseData(F_TYPE x[], int a);
int __declspec(dllexport) FAR PASCAL ScaleData(F_TYPE z[], F_TYPE add,
    F_TYPE mult, BOOL datype);
int __declspec(dllexport) FAR PASCAL NormalizeData(F_TYPE z[], F_TYPE avg,
    BOOL datype);
    
// Frequency offset functions
int __declspec(dllexport) FAR PASCAL RemoveFreqOffset(F_TYPE x[], F_TYPE a,
    F_TYPE b, BOOL bA);
int __declspec(dllexport) FAR PASCAL CalcFreqOffset(F_TYPE x[], F_TYPE *p_a,
    F_TYPE *p_b, F_TYPE *p_v);
int __declspec(dllexport) FAR PASCAL CalcFirstDiff(F_TYPE x[], F_TYPE *p_d);
    
// Outlier functions
int __declspec(dllexport) FAR PASCAL FindFreqOutliers(F_TYPE y[],
    float sig_mult, BOOL remove_flag);

// Noise ID functions   
int __declspec(dllexport) FAR PASCAL NoiseID(int nNum, int nSigmaType,
    int nAvgFactor, float fBW, float fRatio);

// Noise generation functions
int __declspec(dllexport) FAR PASCAL GenNoise(int n_pts, double x[],
	float Q_d, float b,	int *idum, int add_flag);

// Spectral functions
int __declspec(dllexport) FAR PASCAL MultiTaperSpectrumCalc(F_TYPE y[],
	float data[], float freq[], float dt, int num_points, float npi,
	int nwin, int kind, int inorm, int win_type, int p_type,
	float corr_factor, BOOL bDataType);
int __declspec(dllexport) FAR PASCAL SpectrumCalc(F_TYPE y[], float xdata[],
	float ydata[], float tau, int n, int win_num, int win_type,
	int phase_noise_type, float correction_factor, int af, int data_type);

// Date functions
long __declspec(dllexport) FAR PASCAL DateToMJD(int day, int month,
    int year);
void __declspec(dllexport) FAR PASCAL DateConv(int *lpnMonth,
    int *lpnDaysInMonth, int *lpnYear, int *lpnDOW, int *lpnDOY,
    long *lplnMJD);
int __declspec(dllexport) FAR PASCAL DayOfWeek(long gdate);
void __declspec(dllexport) FAR PASCAL BreakDate(long gdate, int *mo,
    int *da, int *yr);
long __declspec(dllexport) FAR PASCAL DateToJulian(long gdate);
long __declspec(dllexport) FAR PASCAL MakeDate(int mo, int da, int yr);
long __declspec(dllexport) FAR PASCAL JulianToDate(long jdate);
int __declspec(dllexport) FAR PASCAL MJDtoGPS(long MJD);
int __declspec(dllexport) FAR PASCAL MJDtoDOY(long lnMJD);
void __declspec(dllexport) FAR PASCAL MJDToDate(long mjd, int *day,
    int *month, int *year);

/***************************************************************************/
